@@ -18,6 +18,9 @@
 #include <stdio.h>
 #include <string.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include "transport.h"
 #include "format.h"
 #include "i18n.h"
