@@ -5,10 +5,11 @@
  **
  ** Copyright 2000-2002, Michael Smith <msmith@xiph.org>
  **
- ** Portions from Vorbize, (c) Kenneth Arnold <kcarnold@yahoo.com>
+ ** Portions from Vorbize, (c) Kenneth Arnold <kcarnold-xiph@arnoldnet.net>
  ** and libvorbis examples, (c) Monty <monty@xiph.org>
  **/
 
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
@@ -19,7 +20,6 @@
 #include "encode.h"
 #include "i18n.h"
 
-
 #define READSIZE 1024
 
 
@@ -46,46 +46,41 @@ int oe_write_page(ogg_page *page, FILE *fp);
 static void set_advanced_encoder_options(adv_opt *opts, int count, 
         vorbis_info *vi)
 {
-    int hard = 0;
-    int avg = 0;
-    struct ovectl_ratemanage_arg ai;
+    int manage = 0;
+    struct ovectl_ratemanage2_arg ai;
     int i;
     double dval;
     long lval;
 
-    vorbis_encode_ctl(vi, OV_ECTL_RATEMANAGE_GET, &ai);
+    vorbis_encode_ctl(vi, OV_ECTL_RATEMANAGE2_GET, &ai);
 
     for(i=0; i < count; i++) {
         fprintf(stderr, _("Setting advanced encoder option \"%s\" to %s\n"),
                 opts[i].arg, opts[i].val);
 
-        if(!strcmp(opts[i].arg, "bitrate_average_window")) {
-            SETD(ai.bitrate_av_window);
-            avg = 1;
+        if(!strcmp(opts[i].arg, "bitrate_average_damping")) {
+            SETD(ai.bitrate_average_damping);
+            manage = 1;
         }
-        else if(!strcmp(opts[i].arg, "bitrate_average_window_center")) {
-            SETD(ai.bitrate_av_window_center);
-            avg = 1;
+        else if(!strcmp(opts[i].arg, "bitrate_average")) {
+            SETL(ai.bitrate_average_kbps);
+            manage = 1;
         }
-        else if(!strcmp(opts[i].arg, "bitrate_average_low")) {
-            SETL(ai.bitrate_av_lo);
-            avg = 1;
+        else if(!strcmp(opts[i].arg, "bit_reservoir_bias")) {
+            SETD(ai.bitrate_limit_reservoir_bias);
+            manage = 1;
         }
-        else if(!strcmp(opts[i].arg, "bitrate_average_high")) {
-            SETL(ai.bitrate_av_hi);
-            avg = 1;
+        else if(!strcmp(opts[i].arg, "bit_reservoir_bits")) {
+            SETL(ai.bitrate_limit_reservoir_bits);
+            manage = 1;
         }
         else if(!strcmp(opts[i].arg, "bitrate_hard_min")) {
-            SETL(ai.bitrate_hard_min);
-            hard = 1;
+            SETL(ai.bitrate_limit_min_kbps);
+            manage = 1;
         }
         else if(!strcmp(opts[i].arg, "bitrate_hard_max")) {
-            SETL(ai.bitrate_hard_max);
-            hard = 1;
-        }
-        else if(!strcmp(opts[i].arg, "bitrate_hard_window")) {
-            SETD(ai.bitrate_hard_window);
-            hard = 1;
+            SETL(ai.bitrate_limit_max_kbps);
+            manage = 1;
         }
         else if(!strcmp(opts[i].arg, "impulse_noisetune")) {
             double val;
@@ -105,10 +100,11 @@ static void set_advanced_encoder_options(adv_opt *opts, int count,
         }
     }
 
-    if(hard)
-        vorbis_encode_ctl(vi, OV_ECTL_RATEMANAGE_HARD, &ai);
-    if(avg)
-        vorbis_encode_ctl(vi, OV_ECTL_RATEMANAGE_AVG, &ai);
+    if(manage) {
+        if(vorbis_encode_ctl(vi, OV_ECTL_RATEMANAGE2_SET, &ai)) {
+            fprintf(stderr, "Failed to set advanced rate management parameters\n");
+        }
+    }
 }
 
 int oe_encode(oe_enc_opt *opt)
@@ -161,17 +157,50 @@ int oe_encode(oe_enc_opt *opt)
 	        return 1;
         }
 
-        /* do we have optional hard quality restrictions? */
+        /* do we have optional hard bitrate restrictions? */
         if(opt->max_bitrate > 0 || opt->min_bitrate > 0){
-            struct ovectl_ratemanage_arg ai;
-	        vorbis_encode_ctl(&vi, OV_ECTL_RATEMANAGE_GET, &ai);
+            struct ovectl_ratemanage2_arg ai;
+	        vorbis_encode_ctl(&vi, OV_ECTL_RATEMANAGE2_GET, &ai);
+
+            /* libvorbis 1.1 (and current svn) doesn't actually fill this in,
+               which looks like a bug. It'll then reject it when we call the
+               SET version below. So, fill it in with the values that libvorbis
+               would have used to fill in this structure if we were using the
+               bitrate-oriented setup functions. Unfortunately, some of those
+               values are dependent on the bitrate, and libvorbis has no way to
+               get a nominal bitrate from a quality value. Well, except by doing
+               a full setup... So, we do that. 
+               Also, note that this won't work correctly unless you have a very
+               recent (2005/03/04 or later) version of libvorbis from svn).
+             */
+            long bitrate;
+
+            {
+                vorbis_info vi2;
+                vorbis_info_init(&vi2);
+                vorbis_encode_setup_vbr(&vi2, opt->channels, opt->rate, opt->quality);
+                vorbis_encode_setup_init(&vi2);
+                bitrate = vi2.bitrate_nominal;
+                vorbis_info_clear(&vi2);
+            }
+
+            ai.bitrate_average_kbps = bitrate/1000;
+            ai.bitrate_average_damping = 1.5;
+            ai.bitrate_limit_reservoir_bits = bitrate * 2;
+            ai.bitrate_limit_reservoir_bias = .1;
 	
-	        ai.bitrate_hard_min=opt->min_bitrate;
-	        ai.bitrate_hard_max=opt->max_bitrate;
+            /* And now the ones we actually wanted to set */
+	        ai.bitrate_limit_min_kbps=opt->min_bitrate;
+	        ai.bitrate_limit_max_kbps=opt->max_bitrate;
 	        ai.management_active=1;
 
-	        vorbis_encode_ctl(&vi, OV_ECTL_RATEMANAGE_SET, &ai);
-
+	        if(vorbis_encode_ctl(&vi, OV_ECTL_RATEMANAGE2_SET, &ai) == 0)
+                fprintf(stderr, _("Set optional hard quality restrictions\n"));
+            else {
+                fprintf(stderr, _("Failed to set bitrate min/max in quality mode\n"));
+                vorbis_info_clear(&vi);
+                return 1;
+            }
         }
 
 
@@ -188,12 +217,15 @@ int oe_encode(oe_enc_opt *opt)
     
     if(opt->managed && opt->bitrate < 0)
     {
-        vorbis_encode_ctl(&vi, OV_ECTL_RATEMANAGE_AVG, NULL);
+      struct ovectl_ratemanage2_arg ai;
+      vorbis_encode_ctl(&vi, OV_ECTL_RATEMANAGE2_GET, &ai);
+      ai.bitrate_average_kbps=-1;
+      vorbis_encode_ctl(&vi, OV_ECTL_RATEMANAGE2_SET, &ai);
     }
     else if(!opt->managed)
     {
         /* Turn off management entirely (if it was turned on). */
-        vorbis_encode_ctl(&vi, OV_ECTL_RATEMANAGE_SET, NULL);
+        vorbis_encode_ctl(&vi, OV_ECTL_RATEMANAGE2_SET, NULL);
     }
     
     set_advanced_encoder_options(opt->advopt, opt->advopt_count, &vi);
@@ -346,7 +378,7 @@ void update_statistics_full(char *fn, long total, long done, double time)
 	seconds = (int)(remain_time - (double)((int)remain_time/60)*60);
 
 	fprintf(stderr, "\r");
-	fprintf(stderr, _("\t[%5.1f%%] [%2dm%.2ds remaining] %c"), 
+	fprintf(stderr, _("\t[%5.1f%%] [%2dm%.2ds remaining] %c "), 
 			done*100.0/total, minutes, seconds, spinner[spinpoint++%4]);
 }
 
@@ -356,7 +388,7 @@ void update_statistics_notime(char *fn, long total, long done, double time)
 	static int spinpoint =0;
 	
 	fprintf(stderr, "\r");
-	fprintf(stderr, _("\tEncoding [%2dm%.2ds so far] %c"), 
+	fprintf(stderr, _("\tEncoding [%2dm%.2ds so far] %c "), 
             ((int)time)/60, (int)(time - (double)((int)time/60)*60),
 			spinner[spinpoint++%4]);
 }
